@@ -1,6 +1,22 @@
 
 module HLint.Default where
 
+import Control.Arrow
+import Control.Exception
+import Control.Monad
+import Data.Function
+import Data.Int
+import Data.List as X
+import Data.Maybe
+import Data.Monoid
+import Data.Ord
+import System.IO
+
+import IO as System.IO
+import List as Data.List
+import Maybe as Data.Maybe
+import Monad as Control.Monad
+
 -- I/O
 
 error = putStrLn (show x) ==> print x
@@ -15,12 +31,12 @@ error = hPrint stdout ==> print
 
 -- ORD
 
-error = not (a == b) ==> a /= b
-error = not (a /= b) ==> a == b
-error = not (a >  b) ==> a <= b
-error = not (a >= b) ==> a <  b
-error = not (a <  b) ==> a >= b
-error = not (a <= b) ==> a >  b
+error = not (a == b) ==> a /= b where note = "incorrect if either value is NaN"
+error = not (a /= b) ==> a == b where note = "incorrect if either value is NaN"
+error = not (a >  b) ==> a <= b where note = "incorrect if either value is NaN"
+error = not (a >= b) ==> a <  b where note = "incorrect if either value is NaN"
+error = not (a <  b) ==> a >= b where note = "incorrect if either value is NaN"
+error = not (a <= b) ==> a >  b where note = "incorrect if either value is NaN"
 error = compare x y /= GT ==> x <= y
 error = compare x y == LT ==> x < y
 error = compare x y /= LT ==> x >= y
@@ -47,8 +63,6 @@ warn = concat [a,b] ==> a ++ b
 error "Use map once" = map f (map g x) ==> map (f . g) x
 warn  = x !! 0 ==> head x
 error = take n (repeat x) ==> replicate n x
-error = x ++ concatMap (' ':) y ==> unwords (x:y)
-error = concat (intersperse " " x) ==> unwords x
 error = head (reverse x) ==> last x
 error = head (drop n x) ==> x !! n
 error = reverse (tail (reverse x)) ==> init x
@@ -61,12 +75,19 @@ error = or (map p x) ==> any p x
 error = and (map p x) ==> all p x
 error = zipWith (,) ==> zip
 error = zipWith3 (,,) ==> zip3
-warn  = length x == 0 ==> null x
+warn  = length x == 0 ==> null x where note = "increases laziness"
 warn  "Use null" = length x /= 0 ==> not (null x)
 error "Use :" = (\x -> [x]) ==> (:[])
 error = map (uncurry f) (zip x y) ==> zipWith f x y
 error = not (elem x y) ==> notElem x y
 warn  = foldr f z (map g x) ==> foldr (f . g) z x
+error = x ++ concatMap (' ':) y ==> unwords (x:y)
+error = intercalate " " ==> unwords
+warn  = concat (intersperse x y) ==> intercalate x y where _ = notEq x " "
+warn  = concat (intersperse " " x) ==> unwords x
+error "Use any" = null (filter f x) ==> not (any f x)
+error "Use any" = filter f x == [] ==> not (any f x)
+error = filter f x /= [] ==> any f x
 
 -- FOLDS
 
@@ -83,8 +104,8 @@ error = foldl (*) 1 ==> product
 error = (\x -> x) ==> id
 error = (\(_,y) -> y) ==> snd
 error = (\(x,_) -> x) ==> fst
-error = (\x y-> f (x,y)) ==> curry f where _ = notIn [x,y] f
-error = (\(x,y) -> f x y) ==> uncurry f where _ = notIn [x,y] f
+warn "Use curry" = (\x y-> f (x,y)) ==> curry f where _ = notIn [x,y] f
+warn "Use uncurry" = (\(x,y) -> f x y) ==> uncurry f where _ = notIn [x,y] f
 error "Redundant $" = (($) . f) ==> f
 error "Redundant $" = (f $) ==> f
 warn  = (\x -> y) ==> const y where _ = isAtom y && notIn x y
@@ -111,6 +132,7 @@ warn  "Use if" = case a of {False -> f; _ -> t} ==> if a then t else f
 
 error = id *** g ==> second g
 error = f *** id ==> first f
+error = zip (map f x) (map g x) ==> map (f Control.Arrow.&&& g) x
 warn  = (\(x,y) -> (f x, g y)) ==> f Control.Arrow.*** g where _ = notIn [x,y] [f,g]
 warn  = (\x -> (f x, g x)) ==> f Control.Arrow.&&& g where _ = notIn x [f,g]
 warn  = (\(x,y) -> (f x,y)) ==> Control.Arrow.first f where _ = notIn [x,y] f
@@ -126,8 +148,10 @@ error "Functor law" = fmap id ==> id
 error "Monad law, left identity" = return a >>= f ==> f a
 error "Monad law, right identity" = m >>= return ==> m
 warn  = m >>= return . f ==> fmap f m
-error = (if x then y else return ()) ==> Control.Monad.when x $ _noParen_ y
-error = (if x then return () else y) ==> Control.Monad.unless x $ _noParen_ y
+error = (if x then y else return ()) ==> Control.Monad.when x $ _noParen_ y where _ = not (isAtom y)
+error = (if x then y else return ()) ==> Control.Monad.when x y where _ = isAtom y
+error = (if x then return () else y) ==> Control.Monad.unless x $ _noParen_ y where _ = not (isAtom y)
+error = (if x then return () else y) ==> Control.Monad.unless x y where _ = isAtom y
 error = sequence (map f x) ==> mapM f x
 error = sequence_ (map f x) ==> mapM_ f x
 warn  = flip mapM ==> Control.Monad.forM
@@ -149,6 +173,7 @@ error = mapM_ f (map g x) ==> mapM_ (f . g) x
 -- LIST COMP
 
 warn  "Use list comprehension" = (if b then [x] else []) ==> [x | b]
+warn  "Redundant list comprehension" = [x | x <- y] ==> y where _ = isVar x
 
 -- SEQ
 
@@ -171,19 +196,22 @@ warn  = (case x of Nothing -> y; Just a -> a)  ==> fromMaybe y x
 warn  = (case x of Just a -> a; Nothing -> y)  ==> fromMaybe y x
 error = (if isNothing x then y else fromJust x) ==> fromMaybe y x
 error = (if isJust x then fromJust x else y) ==> fromMaybe y x
+error = isJust x && (fromJust x == y) ==> x == Just y
 
 -- INFIX
 
-warn "Use infix" = elem x y ==> x `elem` y where _ = not (isInfixApp original) && not (isParen result)
-warn "Use infix" = notElem x y ==> x `notElem` y where _ = not (isInfixApp original) && not (isParen result)
-warn "Use infix" = isInfixOf x y ==> x `isInfixOf` y where _ = not (isInfixApp original) && not (isParen result)
-warn "Use infix" = isSuffixOf x y ==> x `isSuffixOf` y where _ = not (isInfixApp original) && not (isParen result)
-warn "Use infix" = isPrefixOf x y ==> x `isPrefixOf` y where _ = not (isInfixApp original) && not (isParen result)
-warn "Use infix" = union x y ==> x `union` y where _ = not (isInfixApp original) && not (isParen result)
-warn "Use infix" = intersect x y ==> x `intersect` y where _ = not (isInfixApp original) && not (isParen result)
+warn "Use infix" = X.elem x y ==> x `X.elem` y where _ = not (isInfixApp original) && not (isParen result)
+warn "Use infix" = X.notElem x y ==> x `X.notElem` y where _ = not (isInfixApp original) && not (isParen result)
+warn "Use infix" = X.isInfixOf x y ==> x `X.isInfixOf` y where _ = not (isInfixApp original) && not (isParen result)
+warn "Use infix" = X.isSuffixOf x y ==> x `X.isSuffixOf` y where _ = not (isInfixApp original) && not (isParen result)
+warn "Use infix" = X.isPrefixOf x y ==> x `X.isPrefixOf` y where _ = not (isInfixApp original) && not (isParen result)
+warn "Use infix" = X.union x y ==> x `X.union` y where _ = not (isInfixApp original) && not (isParen result)
+warn "Use infix" = X.intersect x y ==> x `X.intersect` y where _ = not (isInfixApp original) && not (isParen result)
 
 -- MATHS
 
+error "Redundant fromIntegral" = fromIntegral x ==> x where _ = isLitInt x
+error "Redundant fromInteger" = fromInteger x ==> x where _ = isLitInt x
 warn  = x + negate y ==> x - y
 warn  = 0 - x ==> negate x
 warn  = log y / log x ==> logBase x y
@@ -194,8 +222,19 @@ warn  = n `rem` 2 == 0 ==> even n
 warn  = n `rem` 2 /= 0 ==> odd n
 warn  = not (even x) ==> odd x
 warn  = not (odd x) ==> even x
+warn  = x ** 0.5 ==> sqrt x
+warn  = x ^^ y ==> x ** y where _ = isLitInt y
 warn  "Use 1" = x ^ 0 ==> 1
 
+-- EXCEPTION
+
+error "Use Control.Exception.catch" = Prelude.catch ==> Control.Exception.catch where note = "Prelude.catch does not catch most exceptions"
+warn = flip Control.Exception.catch ==> handle
+warn = flip (catchJust p) ==> handleJust p
+warn = Control.Exception.bracket b (const a) (const t) ==> Control.Exception.bracket_ b a t
+warn = Control.Exception.bracket (openFile x y) hClose ==> withFile x y
+warn = Control.Exception.bracket (openBinaryFile x y) hClose ==> withBinaryFile x y
+
 -- EVALUATE
 
 -- TODO: These should be moved in to HSE\Evaluate.hs and applied
@@ -231,7 +270,6 @@ error "Evaluate" = unwords [] ==> ""
 error "Evaluate" = x - 0 ==> x
 error "Evaluate" = x * 1 ==> x
 error "Evaluate" = x / 1 ==> x
-error "Evaluate" = id x ==> x
 error "Evaluate" = concat [a] ==> a
 error "Evaluate" = concat [] ==> []
 
@@ -246,6 +284,9 @@ warn  = (do a <- f; g a) ==> f >>= g
     where _ = (isAtom f || isApp f) && notIn a g
 -}
 
+test = hints named test are to allow people to put test code within hint files
+testPrefix = and any prefix also works
+
 
 {-
 <TEST>
@@ -279,6 +320,7 @@ yes = if nullPS s then return False else if headPS s /= '\n' then return False e
     -- if nullPS s || (headPS s /= '\n') then return False else alter_input tailPS >> return True
 yes = if foo then do stuff; moreStuff; lastOfTheStuff else return () \
     -- Control.Monad.when foo $ do stuff ; moreStuff ; lastOfTheStuff
+yes = if foo then stuff else return () -- Control.Monad.when foo stuff
 yes = foo $ \(a, b) -> (a, y + b) -- Control.Arrow.second ((+) y)
 no  = foo $ \(a, b) -> (a, a + b)
 yes = map (uncurry (+)) $ zip [1 .. 5] [6 .. 10] -- zipWith (+) [1 .. 5] [6 .. 10]
@@ -302,10 +344,23 @@ yes x = case x of {True -> a ; False -> b} -- if x then a else b
 yes x = case x of {False -> a ; _ -> b} -- if x then b else a
 no = const . ok . toResponse $ "saved"
 yes = case x z of Nothing -> y z; Just pattern -> pattern -- fromMaybe (y z) (x z)
-yes = if p then s else return () -- Control.Monad.when p $ s
+yes = if p then s else return () -- Control.Monad.when p s
 error = a $$$$ b $$$$ c ==> a . b $$$$$ c
 yes = when (not . null $ asdf) -- unless (null asdf)
-
+yes = id 1 -- 1
+yes = case concat (map f x) of [] -> [] -- concatMap f x
+yes = Map.union a b -- a `Map.union` b
+yes = [v | v <- xs] -- xs
+no  = [Left x | Left x <- xs]
+yes = Map.union a b -- a `Map.union` b
+when p s = if p then s else return ()
+yes = x ^^ 18 -- x ** 18
+no = x ^^ 18.5
+instance Arrow (->) where first f = f *** id
+yes = fromInteger 12 -- 12
+yes = catch -- Control.Exception.catch
+import Prelude hiding (catch); no = catch
+import Control.Exception as E; no = E.catch
 
 import Prelude \
 yes = flip mapM -- Control.Monad.forM
@@ -317,5 +372,19 @@ import Control.Monad(forM_) \
 yes = flip mapM -- Control.Monad.forM
 import qualified Control.Monad \
 yes = flip mapM -- Control.Monad.forM
+import qualified Control.Monad as CM \
+yes = flip mapM -- CM.forM
+import qualified Control.Monad as CM(forM,filterM) \
+yes = flip mapM -- CM.forM
+import Control.Monad as CM(forM,filterM) \
+yes = flip mapM -- forM
+import Control.Monad hiding (forM) \
+yes = flip mapM -- Control.Monad.forM
+import Control.Monad hiding (filterM) \
+yes = flip mapM -- forM
+import qualified Data.Text.Lazy as DTL \
+main = DTL.concat $ map (`DTL.snoc` '-') [DTL.pack "one", DTL.pack "two", DTL.pack "three"]
+import Text.Blaze.Html5.Attributes as A \
+main = A.id (stringValue id')
 </TEST>
 -}
