@@ -1,6 +1,6 @@
 {-# LANGUAGE PatternGuards #-}
 
-module CmdLine(Cmd(..), getCmd, exitWithHelp) where
+module CmdLine(Cmd(..), CppFlags(..), getCmd, exitWithHelp) where
 
 import Control.Monad
 import Data.List
@@ -17,17 +17,25 @@ import Paths_hlint
 import Data.Version
 
 
+data CppFlags
+    = NoCpp
+    | CppSimple
+    | Cpphs CpphsOptions
+
+
+-- FIXME: Hints vs GivenHints is horrible
 data Cmd = Cmd
     {cmdTest :: Bool                 -- ^ run in test mode?
-    ,cmdFiles :: [FilePath]          -- ^ which files to run it on
+    ,cmdFiles :: Maybe [FilePath]    -- ^ which files to run it on, nothing = none given
     ,cmdHintFiles :: [FilePath]      -- ^ which settingsfiles to use
+    ,cmdGivenHints :: [FilePath]     -- ^ which settignsfiles were explicitly given
     ,cmdReports :: [FilePath]        -- ^ where to generate reports
     ,cmdIgnore :: [String]           -- ^ the hints to ignore
     ,cmdShowAll :: Bool              -- ^ display all skipped items
     ,cmdColor :: Bool                -- ^ color the result
-    ,cmdCpphs :: Maybe CpphsOptions  -- ^ options for cpphs
+    ,cmdCpp :: CppFlags              -- ^ options for CPP
     ,cmdDataDir :: FilePath          -- ^ the data directory
-    ,cmdEncoding :: String           -- ^ the text encoding
+    ,cmdEncoding :: Encoding           -- ^ the text encoding
     ,cmdFindHints :: [FilePath]      -- ^ source files to look for hints in
     ,cmdLanguage :: [Extension]      -- ^ the extensions (may be prefixed by "No")
     ,cmdQuiet :: Bool                -- ^ supress all console output
@@ -44,6 +52,7 @@ data Opts = Help
           | Color
           | Define String
           | Include String
+          | SimpleCpp
           | Ext String
           | DataDir String
           | Encoding String
@@ -70,6 +79,7 @@ opts = [Option "?" ["help"] (NoArg Help) "Display help message"
        ,Option "q" ["quiet"] (NoArg Quiet) "Supress most console output"
        ,Option ""  ["cpp-define"] (ReqArg Define "name[=value]") "CPP #define"
        ,Option ""  ["cpp-include"] (ReqArg Include "dir") "CPP include path"
+       ,Option ""  ["cpp-simple"] (NoArg SimpleCpp) "Use a simple CPP (strip # lines)"
        ]
 
 
@@ -92,32 +102,36 @@ getCmd args = do
 
     let exts = [x | Ext x <- opt]
         exts2 = if null exts then ["hs","lhs"] else exts
-    files <- concatMapM (getFile exts2) files
+    files <- if null files then return Nothing else fmap Just $ concatMapM (getFile exts2) files
     findHints <- concatMapM (getFile exts2) [x | FindHints x <- opt]
 
     let hintFiles = [x | Hints x <- opt]
     hints <- mapM (getHintFile dataDir) $ hintFiles ++ ["HLint" | null hintFiles]
+    let givenHints = if null hintFiles then [] else hints
+
+    let languages = getExtensions [x | Language x <- opt]
 
     let cpphs = defaultCpphsOptions
             {boolopts=defaultBoolOptions{hashline=False}
             ,includes = [x | Include x <- opt]
             ,defines = [(a,drop 1 b) | Define x <- opt, let (a,b) = break (== '=') x]
             }
+    let cpp | SimpleCpp `elem` opt = CppSimple -- must be first, so can disable CPP
+            | CPP `elem` languages = Cpphs cpphs
+            | otherwise = NoCpp
 
-    let encoding = last $ "" : [x | Encoding x <- opt]
-    when (encoding /= "") $ warnEncoding encoding
-
-    let languages = getExtensions [x | Language x <- opt]
+    encoding <- newEncoding $ last $ "" : [x | Encoding x <- opt]
 
     return Cmd
         {cmdTest = test
         ,cmdFiles = files
         ,cmdHintFiles = hints
+        ,cmdGivenHints = givenHints
         ,cmdReports = [x | Report x <- opt]
         ,cmdIgnore = [x | Skip x <- opt]
         ,cmdShowAll = ShowAll `elem` opt
         ,cmdColor = Color `elem` opt
-        ,cmdCpphs = if CPP `elem` languages then Just cpphs else Nothing
+        ,cmdCpp = cpp
         ,cmdDataDir = dataDir
         ,cmdEncoding = encoding
         ,cmdFindHints = findHints
@@ -133,7 +147,7 @@ exitWithHelp = do
 
 
 versionText :: String
-versionText = "HLint v" ++ showVersion version ++ ", (C) Neil Mitchell 2006-2010\n"
+versionText = "HLint v" ++ showVersion version ++ ", (C) Neil Mitchell 2006-2011\n"
 
 
 helpText :: String
