@@ -1,9 +1,10 @@
 {-# LANGUAGE RecordWildCards #-}
 
-module HLint(hlint, Suggestion, suggestionLocation) where
+module HLint(hlint, Suggestion, suggestionLocation, suggestionSeverity, Severity(..)) where
 
 import Control.Monad
 import Data.List
+import Data.Maybe
 
 import CmdLine
 import Settings
@@ -28,6 +29,11 @@ suggestionLocation :: Suggestion -> SrcLoc
 suggestionLocation = loc . fromSuggestion
 
 
+-- | From a suggestion, determine how severe it is.
+suggestionSeverity :: Suggestion -> Severity
+suggestionSeverity = severity . fromSuggestion
+
+
 
 -- | This function takes a list of command line arguments, and returns the given suggestions.
 --   To see a list of arguments type @hlint --help@ at the console.
@@ -40,13 +46,15 @@ suggestionLocation = loc . fromSuggestion
 hlint :: [String] -> IO [Suggestion]
 hlint args = do
     cmd@Cmd{..} <- getCmd args
-    let flags = parseFlags{cpphs=cmdCpphs, encoding=cmdEncoding, language=cmdLanguage}
+    let flags = parseFlags{cppFlags=cmdCpp, encoding=cmdEncoding, language=cmdLanguage}
     if cmdTest then
-        test cmdDataDir >> return []
-     else if null cmdFiles && notNull cmdFindHints then
+        test (\x -> hlint x >> return ()) cmdDataDir cmdGivenHints >> return []
+     else if isNothing cmdFiles && notNull cmdFindHints then
         mapM_ (\x -> putStrLn . fst =<< findSettings flags x) cmdFindHints >> return []
-     else if null cmdFiles then
+     else if isNothing cmdFiles then
         exitWithHelp
+     else if cmdFiles == Just [] then
+        error "No files found"
      else
         runHints cmd flags
 
@@ -59,8 +67,8 @@ runHints Cmd{..} flags = do
     settings3 <- return [Classify Ignore x ("","") | x <- cmdIgnore]
     let settings = settings1 ++ settings2 ++ settings3
 
-    ideas <- fmap concat $ parallel [listM' =<< applyHint flags settings x | x <- cmdFiles]
-    let (showideas,hideideas) = partition (\i -> cmdShowAll || rank i /= Ignore) ideas
+    ideas <- fmap concat $ parallel [listM' =<< applyHint flags settings x | x <- fromMaybe [] cmdFiles]
+    let (showideas,hideideas) = partition (\i -> cmdShowAll || severity i /= Ignore) ideas
     showItem <- if cmdColor then showANSI else return show
     mapM_ (outStrLn . showItem) showideas
 
