@@ -5,6 +5,7 @@ module HSE.Util where
 import Control.Monad
 import Data.List
 import Data.Maybe
+import System.FilePath
 import HSE.Type
 import Language.Haskell.Exts.Annotated.Simplify(sOp, sAssoc)
 
@@ -34,6 +35,9 @@ moduleImports (Module _ _ _ x _) = x
 modulePragmas :: Module_ -> [ModulePragma S]
 modulePragmas (Module _ _ x _ _) = x
 
+fromModuleName :: ModuleName S -> String
+fromModuleName (ModuleName _ x) = x
+
 isChar :: Exp_ -> Bool
 isChar (Lit _ Char{}) = True
 isChar _ = False
@@ -74,7 +78,6 @@ isLambda Lambda{} = True; isLambda _ = False
 isMDo MDo{} = True; isMDo _ = False
 isBoxed Boxed{} = True; isBoxed _ = False
 isDerivDecl DerivDecl{} = True; isDerivDecl _ = False
-isFunDep FunDep{} = True; isFunDep _ = False
 isPBangPat PBangPat{} = True; isPBangPat _ = False
 isPExplTypeArg PExplTypeArg{} = True; isPExplTypeArg _ = False
 isPFieldPun PFieldPun{} = True; isPFieldPun _ = False
@@ -105,6 +108,12 @@ unqual :: QName S -> QName S
 unqual (Qual an _ x) = UnQual an x
 unqual x = x
 
+fromQual :: QName S -> Name S
+fromQual (Qual _ _ x) = x
+fromQual (UnQual _ x) = x
+
+isSpecial :: QName S -> Bool
+isSpecial Special{} = True; isSpecial _ = False
 
 isDol :: QOp S -> Bool
 isDol (QVarOp _ (UnQual _ (Symbol _ "$"))) = True
@@ -138,10 +147,24 @@ isLexeme _ = False
 
 getEquations :: Decl s -> [Decl s]
 getEquations (FunBind s xs) = map (FunBind s . (:[])) xs
-getEquations (PatBind s (PVar _ name) _ bod bind) = [FunBind s [Match s name [] bod bind]]
+getEquations x@PatBind{} = [toFunBind x]
 getEquations x = [x]
 
 
+toFunBind :: Decl s -> Decl s
+toFunBind (PatBind s (PVar _ name) _ bod bind) = FunBind s [Match s name [] bod bind]
+toFunBind x = x
+
+
+fromGuardedAlts :: GuardedAlts s -> Rhs s
+fromGuardedAlts (UnGuardedAlt s x) = UnGuardedRhs s x
+fromGuardedAlts (GuardedAlts s xs) = GuardedRhss s [GuardedRhs a b c | GuardedAlt a b c <- xs]
+
+toGuardedAlts :: Rhs s -> GuardedAlts s
+toGuardedAlts (UnGuardedRhs s x) = UnGuardedAlt s x
+toGuardedAlts (GuardedRhss s xs) = GuardedAlts s [GuardedAlt a b c | GuardedRhs a b c <- xs]
+
+
 -- case and if both have branches, nothing else does
 replaceBranches :: Exp s -> ([Exp s], [Exp s] -> Exp s)
 replaceBranches (If s a b c) = ([b,c], \[b,c] -> If s a b c)
@@ -228,7 +251,10 @@ universeParentExp xs = concat [(Nothing, x) : f x | x <- childrenBi xs]
 -- SRCLOC FUNCTIONS
 
 showSrcLoc :: SrcLoc -> String
-showSrcLoc (SrcLoc file line col) = file ++ ":" ++ show line ++ ":" ++ show col ++ ":"
+showSrcLoc (SrcLoc file line col) = take 1 file ++ f (drop 1 file) ++ ":" ++ show line ++ ":" ++ show col
+    where f (x:y:zs) | isPathSeparator x && isPathSeparator y = f $ x:zs
+          f (x:xs) = x : f xs
+          f [] = []
 
 toSrcLoc :: SrcInfo si => si -> SrcLoc
 toSrcLoc = getPointLoc
@@ -250,8 +276,9 @@ dropAnn = fmap (const ())
 
 x /=~= y = not $ x =~= y
 
-elem_ :: (Annotated f, Eq (f ())) => f S -> [f S] -> Bool
+elem_, notElem_ :: (Annotated f, Eq (f ())) => f S -> [f S] -> Bool
 elem_ x = any (x =~=)
+notElem_ x = not . elem_ x
 
 nub_ :: (Annotated f, Eq (f ())) => [f S] -> [f S]
 nub_ = nubBy (=~=)
