@@ -21,11 +21,13 @@ yes = \ x -> (x && x) -- @Warning \x -> x && x
 no = \(x -> y) -> z
 yes = (`foo` (bar baz)) -- @Warning (`foo` bar baz)
 
+
 -- type bracket reduction
 foo :: (Int -> Int) -> Int
 foo :: Int -> (Int -> Int) -- @Warning Int -> Int -> Int
 foo :: (Maybe Int) -> a -- @Warning Maybe Int -> a
 instance Named (DeclHead S)
+data Foo = Foo {foo :: (Maybe Foo)} -- @Warning foo :: Maybe Foo
 
 -- pattern bracket reduction
 foo (True) = 1
@@ -43,6 +45,8 @@ yes = return $ Record{a=b} -- return Record{a=b}
 yes = (b $ c d) ++ e -- b (c d) ++ e
 yes = (a b $ c d) ++ e -- a b (c d) ++ e
 no = (f . g $ a) ++ e
+no = quickCheck ((\h -> cySucc h == succ h) :: Hygiene -> Bool)
+foo = (case x of y -> z; q -> w) :: Int
 </TEST>
 -}
 
@@ -56,7 +60,8 @@ bracketHint :: DeclHint
 bracketHint _ _ x =
     concatMap (\x -> bracket True x ++ dollar x) (childrenBi x :: [Exp_]) ++
     concatMap (bracket False) (childrenBi x :: [Type_]) ++
-    concatMap (bracket False) (childrenBi x :: [Pat_])
+    concatMap (bracket False) (childrenBi x :: [Pat_]) ++
+    concatMap fieldDecl (childrenBi x)
 
 
 bracket :: (Annotated a, Uniplate (a S), ExactP a, Pretty (a S), Brackets (a S)) => Bool -> a S -> [Idea]
@@ -65,6 +70,7 @@ bracket bad = f Nothing
         msg = "Redundant bracket"
 
         -- f (Maybe (index, parent, gen)) child
+        f :: (Annotated a, Uniplate (a S), ExactP a, Pretty (a S), Brackets (a S)) => Maybe (Int,a S,a S -> a S) -> a S -> [Idea]
         f Just{} o@(remParen -> Just x) | isAtom x = err msg o x : g x
         f Nothing o@(remParen -> Just x) | bad = warn msg o x : g x
         f (Just (i,o,gen)) (remParen -> Just x) | not $ needBracket i o x = warn msg o (gen x) : g x
@@ -74,6 +80,12 @@ bracket bad = f Nothing
         g o = concat [f (Just (i,o,gen)) x | (i,(x,gen)) <- zip [0..] $ holes o]
 
 
+fieldDecl :: FieldDecl S -> [Idea]
+fieldDecl o@(FieldDecl a b (UnBangedTy c (TyParen _ d)))
+    = [warn "Redundant bracket" o (FieldDecl a b (UnBangedTy c d))]
+fieldDecl _ = []
+
+
 dollar :: Exp_ -> [Idea]
 dollar = concatMap f . universe
     where
