@@ -25,6 +25,8 @@ sort !f = undefined
 data Set (cxt :: * -> *) a = Set [a]
 {-# LANGUAGE RecordWildCards #-} \
 record field = Record{..}
+{-# LANGUAGE RecordWildCards #-} \
+record = 1 --
 </TEST>
 -}
 
@@ -33,22 +35,38 @@ module Hint.Extensions where
 
 import Hint.Type
 import Data.Maybe
+import Data.List
 import Util
 
 
 extensionsHint :: ModuHint
 extensionsHint _ x = [rawIdea Error "Unused LANGUAGE pragma" (toSrcLoc sl)
-          (prettyPrint o) (if null new then "" else prettyPrint $ LanguagePragma sl new)
+          (prettyPrint o) (if null new then "" else prettyPrint $ LanguagePragma sl $ map (toNamed . showExt) new)
+          (warnings old new)
     | not $ used TemplateHaskell x -- if TH is on, can use all other extensions programmatically
-    , o@(LanguagePragma sl old) <- modulePragmas x
-    , let new = filter (flip used x . classifyExtension . prettyPrint) old
-    , length new /= length old]
+    , o@(LanguagePragma sl exts) <- modulePragmas x
+    , let old = map (classifyExtension . prettyPrint) exts
+    , let new = minimalExtensions x old
+    , sort new /= sort old]
+    where
+        showExt (UnknownExtension x) = x
+        showExt x = show x
+
+
+minimalExtensions :: Module_ -> [Extension] -> [Extension]
+minimalExtensions x es = nub $ concatMap f es
+    where f e = if used e x then [e] else []
+
+
+-- RecordWildCards implies DisambiguateRecordFields, but most people probably don't want it
+warnings old new | RecordWildCards `elem` old && RecordWildCards `notElem` new = "you may need to add DisambiguateRecordFields"
+warnings _ _ = ""
 
 
 used :: Extension -> Module_ -> Bool
 used RecursiveDo = hasS isMDo
 used ParallelListComp = hasS isParComp
-used FunctionalDependencies = hasS isFunDep
+used FunctionalDependencies = hasT (un :: FunDep S)
 used ImplicitParams = hasT (un :: IPName S)
 used EmptyDataDecls = hasS f
     where f (DataDecl _ _ _ _ [] _) = True
